#!/usr/bin/python
# -*- coding:utf-8 -*-

import sys
from struct import unpack
import argparse
import textwrap

# Extract the altitudes on a DEM, from a list of coordinatess
# Copyright (C) 2016 Arthur Delorme - v2.2

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# (contact: delorme@ipgp.fr)

parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description="Extract the altitudes on a DEM, from a list of coordinatess",
        epilog=textwrap.dedent('''
                This program is distributed under the terms of the GNU General Public License.
                This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you
                are welcome to redistribute it under certain conditions.
                (see license.txt for details)
                (contact: delorme@ipgp.fr)
            ''')
    )
parser.add_argument('dem_path', type=str,
        help='path to the DEM (ENVI format; square px; data type: int (16bits) or float (32bits))'
    )
parser.add_argument('coord_path', metavar='coord_list', type=str,
                    help='path to the coordinates list (format: lon,lat)')
parser.add_argument('out_path', type=str, help='path to the output file (format: lon lat alti)')
args = parser.parse_args()

# Read the header
hdr_path = '{}.hdr'.format(args.dem_path)
with open(hdr_path,'r') as f:
    # 1st line must be "ENVI\n"
    if f.readline() != 'ENVI\n':
        sys.exit("Error: the .hdr file is not an ENVI file")
    
    for l in f:
        for to_find in ["file type","data type","samples","map info"]:
            if to_find in l:
                l = l.strip()
                split_l = l.split(' = ')
                # Check: image = ENVI file?
                if to_find == "file type":
                    if split_l[1] != "ENVI Standard":
                        sys.exit("Error: the DEM is not an ENVI file")
                # Check: image = int (16bits) or float (32bits)?
                if to_find == "data type":
                    if split_l[1] == '2':
                        data_type = 'h'
                        data_size = 2
                    elif split_l[1] == '4':
                        data_type = 'f'
                        data_size = 4
                    else:
                        sys.exit("Error: data type")
                # Nb of columns
                if to_find == "samples":
                    col = int(split_l[1])
                # Geo informations
                if to_find == "map info":
                    split_l = split_l[1].split(',')
                    i = 0
                    for to_find2 in split_l:
                        split_l[i] = to_find2.strip()
                        i += 1
                    # Check: square pixel?
                    if split_l[5] != split_l[6]:
                        sys.exit("Error: pixels in the DEM are not squares")
                    # Read values
                    lat_hg = float(split_l[4])
                    lon_hg = float(split_l[3])
                    step = float(split_l[5])
                    # Reference for lat_hg & lon_hg; ex : (1,1) = top-left corner of the 1st pixel
                    loc_hg_x = float(split_l[1]) - 1
                    loc_hg_y = float(split_l[2]) - 1

with open(args.dem_path,'rb') as f_im, open(args.coord_path,'r') as f_coord, open(args.out_path,'w') as f_out:
    # Read coordinates
    while 1:
        l = f_coord.readline()
        if not l:
            break
        l = l.strip()
        if l:
            split_l = l.split(',')
            lat = float(split_l[1])
            lon = float(split_l[0])
            
            # Compute the corresponding image coordinate (X,Y)
            delta_lat = lat_hg - lat
            delta_lon = lon - lon_hg
            x = int(round(delta_lon / step + loc_hg_x))
            y = int(round(delta_lat / step + loc_hg_y))
            
            # Read image
            f_im.seek(data_size*(x+y*col)) # Jump just before the searched pixel
            data = f_im.read(data_size)
            
            f_out.write('{0}\t{1}\t{2}\n'.format(lon, lat, unpack(data_type,data)[0]))

print "File '{0}' was successfully created".format(args.out_path)